@@ -193,8 +193,8 @@ AMP_DISABLE_SSHD=${AMP_DISABLE_SSHD:-0}
 
 AMP_ENABLE_FULL_MAC_SECURITY=${AMP_ENABLE_FULL_MAC_SECURITY:-0}
 
-if [ "$AMP_BASEOS" = "rhel" ] && [ "$AMP_IMAGESIZE" -lt 3 ]; then
-    echo "RHEL based amphora requires an image size of at least 3GB"
+if [ "$AMP_BASEOS" = "rhel" -o "$AMP_BASEOS" = "centos" ] && [ "$AMP_IMAGESIZE" -lt 3 ]; then
+    echo "RHEL/centos based amphora requires an image size of at least 3GB"
     exit 1
 fi
 
