@@ -12,6 +12,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import errno
 import ipaddress
 import logging
 import os
@@ -65,7 +66,13 @@ class BaseOS(object):
         # We need to setup the netns network directory so that the ifup
         # commands used here and in the startup scripts "sees" the right
         # interfaces and scripts.
-        os.makedirs('/etc/netns/' + consts.AMPHORA_NAMESPACE)
+        try:
+            os.makedirs('/etc/netns/' + consts.AMPHORA_NAMESPACE)
+        except OSError as e:
+            # Raise the error if it's not "File exists" otherwise pass
+            if e.errno != errno.EEXIST:
+                raise
+
         shutil.copytree(
             network_dir,
             '/etc/netns/{netns}/{net_dir}'.format(
@@ -136,11 +143,6 @@ class BaseOS(object):
             text = template_port.render(interface=netns_interface)
         else:
             for index, fixed_ip in enumerate(fixed_ips, -1):
-                if index == -1:
-                    netns_ip_interface = netns_interface
-                else:
-                    netns_ip_interface = "{int}:{ip}".format(
-                        int=netns_interface, ip=index)
                 try:
                     ip_addr = fixed_ip['ip_address']
                     cidr = fixed_ip['subnet_cidr']
@@ -157,7 +159,7 @@ class BaseOS(object):
                 except ValueError:
                     return webob.Response(
                         json=dict(message="Invalid network IP"), status=400)
-                new_text = template_port.render(interface=netns_ip_interface,
+                new_text = template_port.render(interface=netns_interface,
                                                 ipv6=ip.version is 6,
                                                 ip_address=ip.exploded,
                                                 broadcast=broadcast,
