@@ -54,8 +54,8 @@ class HealthMonitorController(base.BaseController):
         return db_hm
 
     @wsme_pecan.wsexpose(hm_types.HealthMonitorRootResponse, wtypes.text,
-                         wtypes.text)
-    def get_one(self, id):
+                         [wtypes.text], ignore_extra_args=True)
+    def get_one(self, id, fields=None):
         """Gets a single healthmonitor's details."""
         context = pecan.request.context.get('octavia_context')
         db_hm = self._get_db_hm(context.session, id)
@@ -65,6 +65,8 @@ class HealthMonitorController(base.BaseController):
 
         result = self._convert_db_to_type(
             db_hm, hm_types.HealthMonitorResponse)
+        if fields is not None:
+            result = self._filter_fields([result], fields)[0]
         return hm_types.HealthMonitorRootResponse(healthmonitor=result)
 
     @wsme_pecan.wsexpose(hm_types.HealthMonitorsRootResponse, wtypes.text,
@@ -129,9 +131,8 @@ class HealthMonitorController(base.BaseController):
         try:
             return self.repositories.health_monitor.create(
                 lock_session, **hm_dict)
-        except odb_exceptions.DBDuplicateEntry as de:
-            if ['id'] == de.columns:
-                raise exceptions.IDAlreadyExists()
+        except odb_exceptions.DBDuplicateEntry:
+            raise exceptions.DuplicateHealthMonitor()
         except odb_exceptions.DBError:
             # TODO(blogan): will have to do separate validation protocol
             # before creation or update since the exception messages
@@ -170,17 +171,17 @@ class HealthMonitorController(base.BaseController):
                                    constants.RBAC_POST)
 
         lock_session = db_api.get_session(autocommit=False)
-        if self.repositories.check_quota_met(
-                context.session,
-                lock_session,
-                data_models.HealthMonitor,
-                health_monitor.project_id):
-            lock_session.rollback()
-            raise exceptions.QuotaException
-
-        hm_dict = db_prepare.create_health_monitor(
-            health_monitor.to_dict(render_unsets=True))
         try:
+            if self.repositories.check_quota_met(
+                    context.session,
+                    lock_session,
+                    data_models.HealthMonitor,
+                    health_monitor.project_id):
+                raise exceptions.QuotaException
+
+            hm_dict = db_prepare.create_health_monitor(
+                health_monitor.to_dict(render_unsets=True))
+
             self._test_lb_and_listener_and_pool_statuses(
                 lock_session, health_monitor)
             db_hm = self._validate_create_hm(lock_session, hm_dict)
@@ -245,6 +246,9 @@ class HealthMonitorController(base.BaseController):
         self._auth_validate_action(context, db_hm.project_id,
                                    constants.RBAC_DELETE)
 
+        if db_hm.provisioning_status == constants.DELETED:
+            return
+
         self._test_lb_and_listener_and_pool_statuses(context.session, db_hm)
 
         self.repositories.health_monitor.update(
