@@ -164,6 +164,12 @@ healthmanager_opts = [
     cfg.IntOpt('status_update_threads',
                default=None,
                help=_('Number of processes for amphora status update.')),
+    cfg.IntOpt('health_update_threads',
+               default=None,
+               help=_('Number of processes for amphora health update.')),
+    cfg.IntOpt('stats_update_threads',
+               default=None,
+               help=_('Number of processes for amphora stats update.')),
     cfg.StrOpt('heartbeat_key',
                help=_('key used to validate amphora sending'
                       'the message'), secret=True),
@@ -640,3 +646,10 @@ def handle_deprecation_compatibility():
     if cfg.CONF.api_handler is not None:
         cfg.CONF.set_default('api_handler', cfg.CONF.api_handler,
                              group='api_settings')
+    if cfg.CONF.health_manager.status_update_threads is not None:
+        cfg.CONF.set_default('health_update_threads',
+                             cfg.CONF.health_manager.status_update_threads,
+                             group='health_manager')
+        cfg.CONF.set_default('stats_update_threads',
+                             cfg.CONF.health_manager.status_update_threads,
+                             group='health_manager')
