@@ -26,6 +26,7 @@ PROTOCOL_MAP = {
     constants.PROTOCOL_TCP: 'tcp',
     constants.PROTOCOL_HTTP: 'http',
     constants.PROTOCOL_HTTPS: 'tcp',
+    constants.PROTOCOL_PROXY: 'proxy',
     constants.PROTOCOL_TERMINATED_HTTPS: 'http'
 }
 
@@ -165,6 +166,9 @@ class JinjaTemplater(object):
         # the global value trivial.
         if listener.connection_limit and listener.connection_limit > -1:
             ret_value['global_connection_limit'] = listener.connection_limit
+        else:
+            ret_value['global_connection_limit'] = (
+                constants.HAPROXY_MAX_MAXCONN)
         return ret_value
 
     def _transform_amphora(self, amphora):
@@ -203,11 +207,13 @@ class JinjaTemplater(object):
         }
         if listener.connection_limit and listener.connection_limit > -1:
             ret_value['connection_limit'] = listener.connection_limit
+        else:
+            ret_value['connection_limit'] = constants.HAPROXY_MAX_MAXCONN
         if listener.tls_certificate_id:
             ret_value['default_tls_path'] = '%s.pem' % (
                 os.path.join(self.base_crt_dir,
                              listener.id,
-                             tls_cert.primary_cn))
+                             tls_cert.id))
         if listener.sni_containers:
             ret_value['crt_dir'] = os.path.join(self.base_crt_dir, listener.id)
         if listener.default_pool:
@@ -270,7 +276,9 @@ class JinjaTemplater(object):
             'weight': member.weight,
             'enabled': member.enabled,
             'subnet_id': member.subnet_id,
-            'operating_status': member.operating_status
+            'operating_status': member.operating_status,
+            'monitor_address': member.monitor_address,
+            'monitor_port': member.monitor_port
         }
 
     def _transform_health_monitor(self, monitor):
@@ -311,7 +319,8 @@ class JinjaTemplater(object):
                 l7policy.redirect_pool)
         else:
             ret_value['redirect_pool'] = None
-        l7rules = [self._transform_l7rule(x) for x in l7policy.l7rules]
+        l7rules = [self._transform_l7rule(x) for x in l7policy.l7rules
+                   if x.enabled]
         ret_value['l7rules'] = l7rules
         return ret_value
 
@@ -326,7 +335,8 @@ class JinjaTemplater(object):
             'compare_type': l7rule.compare_type,
             'key': l7rule.key,
             'value': self._escape_haproxy_config_string(l7rule.value),
-            'invert': l7rule.invert
+            'invert': l7rule.invert,
+            'enabled': l7rule.enabled
         }
 
     @staticmethod
