@@ -19,11 +19,13 @@ Defined here so these can also be used at deeper levels than the API.
 """
 
 
+import ipaddress
 import re
 
 import netaddr
 from oslo_config import cfg
 import rfc3986
+import six
 
 from octavia.common import constants
 from octavia.common import exceptions
@@ -55,7 +57,8 @@ def url_path(url_path):
         if (
             p_url.scheme or p_url.userinfo or p_url.host or
             p_url.port or
-            p_url.path is None
+            p_url.path is None or
+            not p_url.path.startswith('/')
         ):
             raise exceptions.InvalidURLPath(url_path=url_path)
     except Exception:
@@ -293,3 +296,37 @@ def is_ip_member_of_cidr(address, cidr):
     if netaddr.IPAddress(address) in netaddr.IPNetwork(cidr):
         return True
     return False
+
+
+def check_session_persistence(SP_dict):
+    try:
+        if SP_dict['cookie_name']:
+            if SP_dict['type'] != constants.SESSION_PERSISTENCE_APP_COOKIE:
+                raise exceptions.ValidationException(detail=_(
+                    'Field "cookie_name" can only be specified with session '
+                    'persistence of type "APP_COOKIE".'))
+            bad_cookie_name = re.compile(r'[\x00-\x20\x22\x28-\x29\x2c\x2f'
+                                         r'\x3a-\x40\x5b-\x5d\x7b\x7d\x7f]+')
+            valid_chars = re.compile(r'[\x00-\xff]+')
+            if (bad_cookie_name.search(SP_dict['cookie_name']) or
+                    not valid_chars.search(SP_dict['cookie_name'])):
+                raise exceptions.ValidationException(detail=_(
+                    'Supplied "cookie_name" is invalid.'))
+        if (SP_dict['type'] == constants.SESSION_PERSISTENCE_APP_COOKIE and
+                not SP_dict['cookie_name']):
+            raise exceptions.ValidationException(detail=_(
+                'Field "cookie_name" must be specified when using the '
+                '"APP_COOKIE" session persistence type.'))
+    except exceptions.ValidationException:
+        raise
+    except Exception:
+        raise exceptions.ValidationException(detail=_(
+            'Invalid session_persistence provided.'))
+
+
+def ip_not_reserved(ip_address):
+    ip_address = (
+        ipaddress.ip_address(six.text_type(ip_address)).exploded.upper())
+    if ip_address in CONF.networking.reserved_ips:
+        raise exceptions.InvalidOption(value=ip_address,
+                                       option='member address')
