@@ -681,6 +681,33 @@ class LoadBalancerRepository(BaseRepository):
             session.add(lb)
             return True
 
+    def set_status_for_failover(self, session, id, status,
+                                raise_exception=False):
+        """Tests and sets a load balancer provisioning status.
+
+        Puts a lock on the load balancer table to check the status of a
+        load balancer.  If the status is ACTIVE or ERROR then the status of
+        the load balancer is updated and the method returns True.  If the
+        status is not ACTIVE, then nothing is done and False is returned.
+
+        :param session: A Sql Alchemy database session.
+        :param id: id of Load Balancer
+        :param status: Status to set Load Balancer if check passes.
+        :param raise_exception: If True, raise ImmutableObject on failure
+        :returns: bool
+        """
+        with session.begin(subtransactions=True):
+            lb = session.query(self.model_class).with_for_update().filter_by(
+                id=id).one()
+            if lb.provisioning_status not in consts.FAILOVERABLE_STATUSES:
+                if raise_exception:
+                    raise exceptions.ImmutableObject(
+                        resource='Load Balancer', id=id)
+                return False
+            lb.provisioning_status = status
+            session.add(lb)
+            return True
+
     def check_load_balancer_expired(self, session, lb_id, exp_age=None):
         """Checks if a given load balancer is expired.
 
@@ -898,21 +925,33 @@ class AmphoraRepository(BaseRepository):
 
         return amp.to_data_model()
 
-    def get_all_lbs_on_amphora(self, session, amphora_id):
+    @staticmethod
+    def get_lb_for_amphora(session, amphora_id):
         """Get all of the load balancers on an amphora.
 
         :param session: A Sql Alchemy database session.
         :param amphora_id: The amphora id to list the load balancers from
         :returns: [octavia.common.data_model]
         """
-        with session.begin(subtransactions=True):
-            lb_subquery = (session.query(self.model_class.load_balancer_id).
-                           filter_by(id=amphora_id).subquery())
-            lb_list = (session.query(models.LoadBalancer).
-                       filter(models.LoadBalancer.id.in_(lb_subquery)).
-                       options(joinedload('*')).all())
-            data_model_list = [model.to_data_model() for model in lb_list]
-            return data_model_list
+        with session.begin():
+            db_lb = (
+                # Get LB records
+                session.query(models.LoadBalancer)
+                # Joined to amphora records
+                .filter(models.LoadBalancer.id ==
+                        models.Amphora.load_balancer_id)
+                # For just this amphora
+                .filter(models.Amphora.id == amphora_id)
+                # Where the amphora is not DELETED
+                .filter(models.Amphora.status != consts.DELETED)
+                # And the LB is also not DELETED
+                .filter(models.LoadBalancer.provisioning_status !=
+                        consts.DELETED)
+                # And what does this do? Some SQLAlchemy magic?
+                .options(joinedload('*'))
+            ).first()
+            if db_lb:
+                return db_lb.to_data_model()
 
     def get_spare_amphora_count(self, session):
         """Get the count of the spare amphora.
